[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# S&amp;P Global Fixed Income Evaluated Prices and Analytics client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)](https://developer.factset.com/api-catalog/sp-global-fixed-income-evaluated-prices-and-analytics-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics/0.1.14)](https://pypi.org/project/fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v/0.1.14)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Gain access to evaluated pricing and analytics data for Corporate Bonds, Municipal Bonds and Asset Backed Securities provided by S&P Global (formerly IHS Markit) .

Data returned through multiple endpoints like prices, spread, yield, yield curve, sensitivity, coupon information, securities metadata and details specific to the Asset Backed Securities.


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 0.1.14
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics==0.1.14
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics==0.1.14
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics
from fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.api import abs_details_api
from fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = abs_details_api.ABSDetailsApi(api_client)
    ids = ["US05522RDC97"] # [str] | Security or Entity identifiers. FactSet Identifiers, tickers, CUSIP, ISIN and SEDOL are accepted as input. <p>***ids limit** = 2000  per request*</p> *<p>Make note, GET Method URL request lines are also limited to a total length of 8192 bytes (8KB). In cases where the service allows for thousands of ids, which may lead to exceeding this request line limit of 8KB, its advised for any requests with large request lines to be requested through the respective \"POST\" method.</p>* 
    start_date = "2021-01-01" # str | The start date requested for a given date range in **YYYY-MM-DD** format. If left blank, the API will default to previous close. Future dates (T+1) are not accepted in this endpoint.  (optional)
    end_date = "2021-12-31" # str | The end date requested for a given date range in **YYYY-MM-DD** format. If left blank, the API will default to previous close. Future dates (T+1) are not accepted in this endpoint.  (optional)
    frequency = "D" # str | Controls the display frequency of the data returned.   * **D** = Daily   * **W** = Weekly, based on the last day of the week of the start date.   * **M** = Monthly, based on the last trading day of the month.   * **AM** = Monthly, based on the start date (e.g., if the start date is June 16, data is displayed for June 16, May 16, April 16 etc.).   * **CQ** = Quarterly based on the last trading day of the calendar quarter (March, June, September, or December).   * **FQ** = Fiscal Quarter of the company.   * **AY** = Actual Annual, based on the start date.   * **CY** = Calendar Annual, based on the last trading day of the calendar year.   * **FY** = Fiscal Annual, based on the last trading day of the company's fiscal year.  (optional) if omitted the server will use the default value of "D"
    calendar = "FIVEDAY" # str | Calendar of data returned. SEVENDAY includes weekends. LOCAL calendar will default to the securities' trading calendar which excludes date records for respective holiday periods. (optional) if omitted the server will use the default value of "FIVEDAY"

    try:
        # Get Asset Backed Security Details for a list of securities
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_abs_details(ids, start_date=start_date, end_date=end_date, frequency=frequency, calendar=calendar)

        pprint(api_response)
    except fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.ApiException as e:
        print("Exception when calling ABSDetailsApi->get_abs_details: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Get Asset Backed Security Details for a list of securities
    #     api_response, http_status_code, response_headers = api_instance.get_abs_details_with_http_info(ids, start_date=start_date, end_date=end_date, frequency=frequency, calendar=calendar)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.ApiException as e:
    #     print("Exception when calling ABSDetailsApi->get_abs_details: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Get Asset Backed Security Details for a list of securities
    #     async_result = api_instance.get_abs_details_async(ids, start_date=start_date, end_date=end_date, frequency=frequency, calendar=calendar)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.ApiException as e:
    #     print("Exception when calling ABSDetailsApi->get_abs_details: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Get Asset Backed Security Details for a list of securities
    #     async_result = api_instance.get_abs_details_with_http_info_async(ids, start_date=start_date, end_date=end_date, frequency=frequency, calendar=calendar)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.ApiException as e:
    #     print("Exception when calling ABSDetailsApi->get_abs_details: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics

configuration = fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics

configuration = fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics

configuration = fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*ABSDetailsApi* | [**get_abs_details**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/ABSDetailsApi.md#get_abs_details) | **GET** /markit-bond-prices-and-analytics/v1/abs-details | Get Asset Backed Security Details for a list of securities
*ABSDetailsApi* | [**get_abs_details_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/ABSDetailsApi.md#get_abs_details_for_list) | **POST** /markit-bond-prices-and-analytics/v1/abs-details | Request Asset Backed Security Details for a list of securities
*CouponApi* | [**get_bond_coupon**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/CouponApi.md#get_bond_coupon) | **GET** /markit-bond-prices-and-analytics/v1/coupon | Get coupon data for given date range and list of securities
*CouponApi* | [**get_bond_coupon_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/CouponApi.md#get_bond_coupon_for_list) | **POST** /markit-bond-prices-and-analytics/v1/coupon | Request coupon data for given date range and list of securities
*IssuerYieldCurveApi* | [**get_bond_issuer_yield_curve**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/IssuerYieldCurveApi.md#get_bond_issuer_yield_curve) | **GET** /markit-bond-prices-and-analytics/v1/issuer-yield-curve | Get yield curve data for given date range and list of securities
*IssuerYieldCurveApi* | [**get_bond_issuer_yield_curve_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/IssuerYieldCurveApi.md#get_bond_issuer_yield_curve_for_list) | **POST** /markit-bond-prices-and-analytics/v1/issuer-yield-curve | Request yield curve data for given date range and list of securities
*MetaApi* | [**get_bond_meta**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/MetaApi.md#get_bond_meta) | **GET** /markit-bond-prices-and-analytics/v1/meta | Get bond meta data for a list of securities.
*MetaApi* | [**get_bond_meta_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/MetaApi.md#get_bond_meta_for_list) | **POST** /markit-bond-prices-and-analytics/v1/meta | Get bond meta data for a list of securities.
*PricesApi* | [**get_bond_prices**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/PricesApi.md#get_bond_prices) | **GET** /markit-bond-prices-and-analytics/v1/prices | Get Bid, Mid and Ask prices for a list of securities
*PricesApi* | [**get_bond_prices_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/PricesApi.md#get_bond_prices_for_list) | **POST** /markit-bond-prices-and-analytics/v1/prices | Request Bid, Mid and Ask prices for a list of securities
*SensitivityApi* | [**get_bond_sensitivity**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/SensitivityApi.md#get_bond_sensitivity) | **GET** /markit-bond-prices-and-analytics/v1/sensitivity | Get the bond sensitivity data for a list of securities
*SensitivityApi* | [**get_bond_sensitivity_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/SensitivityApi.md#get_bond_sensitivity_for_list) | **POST** /markit-bond-prices-and-analytics/v1/sensitivity | Request the bond sensitivity data for a list of securities
*SpreadApi* | [**get_bond_spread**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/SpreadApi.md#get_bond_spread) | **GET** /markit-bond-prices-and-analytics/v1/spread | Get the Spread and OAS data for a list of securities
*SpreadApi* | [**get_bond_spread_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/SpreadApi.md#get_bond_spread_for_list) | **POST** /markit-bond-prices-and-analytics/v1/spread | Request the Spread and OAS data for a list of securities
*YieldApi* | [**get_bond_yield**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/YieldApi.md#get_bond_yield) | **GET** /markit-bond-prices-and-analytics/v1/yield | Get yield information for given date range and list of securities
*YieldApi* | [**get_bond_yield_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/YieldApi.md#get_bond_yield_for_list) | **POST** /markit-bond-prices-and-analytics/v1/yield | Request yield information for given date range and list of securities


## Documentation For Models

 - [AbsDetails](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/AbsDetails.md)
 - [AbsDetailsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/AbsDetailsRequest.md)
 - [AbsDetailsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/AbsDetailsResponse.md)
 - [BondCoupon](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondCoupon.md)
 - [BondCouponRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondCouponRequest.md)
 - [BondCouponResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondCouponResponse.md)
 - [BondIssuerYieldCurve](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondIssuerYieldCurve.md)
 - [BondIssuerYieldCurveRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondIssuerYieldCurveRequest.md)
 - [BondIssuerYieldCurveResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondIssuerYieldCurveResponse.md)
 - [BondMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondMeta.md)
 - [BondMetaRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondMetaRequest.md)
 - [BondMetaResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondMetaResponse.md)
 - [BondPrices](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondPrices.md)
 - [BondPricesRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondPricesRequest.md)
 - [BondPricesResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondPricesResponse.md)
 - [BondSensitivity](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondSensitivity.md)
 - [BondSensitivityRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondSensitivityRequest.md)
 - [BondSensitivityResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondSensitivityResponse.md)
 - [BondSpread](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondSpread.md)
 - [BondSpreadRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondSpreadRequest.md)
 - [BondSpreadResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondSpreadResponse.md)
 - [BondYield](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondYield.md)
 - [BondYieldRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondYieldRequest.md)
 - [BondYieldResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/BondYieldResponse.md)
 - [Calendar](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/Calendar.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/ErrorResponseSubErrors.md)
 - [Frequency](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/Frequency.md)
 - [Ids](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/Ids.md)
 - [IdsABS](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/SPGlobalFixedIncomeEvaluatedPricesandAnalytics/v1/docs/IdsABS.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.apis and fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.api.default_api import DefaultApi`
- `from fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics
from fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.apis import *
from fds.sdk.SPGlobalFixedIncomeEvaluatedPricesandAnalytics.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

